/*
 * Copyright(C) 2000  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

#include "fat12prj.h"

static const TCHAR BASE_NAME_CFG[] = _TEXT("FAT12");

/*!
 * \ingroup fat12_info_general
 * \brief   .
 * \return  
 * \retval ERROR_SUCCESS  
 * \retval (DWORD)NTE_NO_MEMORY   
 */
DWORD fat12_register(TSupSysContext *context, TSupSysInfo *info)
{
    TFat12Context *ctx;
    TSupSysInfoRegister *inf = (TSupSysInfoRegister*)info;
    DWORD code;
    TCHAR * reg_path = NULL;
    size_t reg_length;

    UNUSED(context);

    SUPSYS_PRE_INFO(info, TSupSysInfoRegister);
    code = support_load_library();
    if (code)
	return code;

    ctx = malloc(sizeof(TFat12Context));
    if (ctx == NULL) {
	support_unload_library();
	return (DWORD)NTE_NO_MEMORY;
    }
    memset(ctx, 0, sizeof(TFat12Context));
    ctx->path_to_item = NULL;
    if (inf->in_group_name.text) {
	TCHAR *ptr = NULL;
	UNUSED(ptr);
	_tcsncpy(ctx->nickname, inf->in_group_name.text, sizeof(ctx->nickname) / sizeof(TCHAR));
	ptr = _tcsrchr(inf->in_group_name.text, _TEXT('_'));
	if (ptr) {
	    ctx->connect[0] = ptr[1];
	    ctx->connect[1] = 0;
	}
	else
	    ctx->connect[1] = 0;
	reg_length = _tcslen(BASE_PATH_CFG) + 1 + _tcslen(BASE_NAME_CFG) + 1 + _tcslen(inf->in_group_name.text) + 1 + _tcslen(DEFAULT_CONNECT) + 1 + _tcslen(LONG_NAMES_KEY) + 1;
	reg_path = malloc(reg_length*sizeof(TCHAR));
	if (!reg_path) {
	    free(ctx);
	    support_unload_library();
	    return (DWORD)NTE_NO_MEMORY;
	}
	_sntprintf(reg_path, reg_length, _TEXT("%s\\%s\\%s\\%s\\%s"), BASE_PATH_CFG, BASE_NAME_CFG, inf->in_group_name.text, DEFAULT_CONNECT, LONG_NAMES_KEY);
    }
    else {
	reg_length = _tcslen(BASE_PATH_CFG) + 1 + _tcslen(BASE_NAME_CFG) + 1 + _tcslen(DEFAULT_CONNECT) + 1 + _tcslen(LONG_NAMES_KEY) + 1;
	reg_path = malloc(reg_length*sizeof(TCHAR));
	if (!reg_path) {
	    free(ctx);
	    support_unload_library();
	    return (DWORD)NTE_NO_MEMORY;
	}
	_sntprintf(reg_path, reg_length, _TEXT("%s\\%s\\%s\\%s"), BASE_PATH_CFG, BASE_NAME_CFG, DEFAULT_CONNECT, LONG_NAMES_KEY);
    }
    ctx->long_names = get_registry_long_names_flag(reg_path);
    free(reg_path);
    ctx->path = NULL;
    ctx->volume_label = NULL;
    ctx->file_name = NULL;
    ctx->folder = NULL;
    ctx->flags = 0;
#ifdef UNIX
    ctx->handle = NULL;
    support_mutex_init(&ctx->lock);
    if (support_mutex_open(&ctx->lock, _TEXT("fat12_lock"), GMUTEX_SNAME)) {
	support_unload_library();
	free(ctx);
	return (DWORD)SCARD_E_READER_UNAVAILABLE;
    }
    ctx->locked = FALSE;
#else /* UNIX */
    ctx->handle = INVALID_HANDLE_VALUE;
    ctx->access = 0;
#endif /* UNIX */
    inf->context = ctx;
    return ERROR_SUCCESS;
}

DWORD fat12_unregister(TSupSysContext *context, TSupSysInfo *info)
{
    TFat12Context *ctx = (TFat12Context*)context;
    UNUSED(info);

    if (ctx) {
#ifdef _WIN32
	LocalFree(ctx->pSD);
#endif //_WIN32
	free(ctx->path);
	free(ctx->path_to_item);
#ifdef UNIX
	support_mutex_close(&ctx->lock);
#endif //UNIX
	free(ctx);
	support_unload_library();
    }
    return ERROR_SUCCESS;
}

BOOL get_registry_long_names_flag(TCHAR *path) {
    long l = 0;
    DWORD code = support_registry_get_long(path, &l);
    if (code || l == 0) return FALSE;
    return TRUE;
}
